using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;

namespace Unity.XR.Pimax.Editor
{
    [CustomEditor(typeof(PvrSettings))]
    public class PvrXRSettingsEditor : UnityEditor.Editor
    {
        private const string StereoRenderingModeAndroid = "stereoRenderingMode";
        private const string ProtectedContext = "protectedContext";
        private const string EyeTracking = "eyeTracking";

        static GUIContent guiStereoRenderingMode = EditorGUIUtility.TrTextContent("Stereo Rendering Mode");
        static GUIContent guiProtectedContext = EditorGUIUtility.TrTextContent("Protected Context");
        static GUIContent guiEyeTracking = EditorGUIUtility.TrTextContent("Eye Tracking");

        private SerializedProperty stereoRenderingMode;
        private SerializedProperty protectedContext;
        private SerializedProperty eyeTracking;

        void OnEnable()
        {
            if (stereoRenderingMode == null)
                stereoRenderingMode = serializedObject.FindProperty(StereoRenderingModeAndroid);
            if (protectedContext == null)
                protectedContext = serializedObject.FindProperty(ProtectedContext);
            if (eyeTracking == null)
                eyeTracking = serializedObject.FindProperty(EyeTracking);

        }

        public override void OnInspectorGUI()
        {
            if (serializedObject == null || serializedObject.targetObject == null)
                return;

            serializedObject.Update();

            BuildTargetGroup selectedBuildTargetGroup = EditorGUILayout.BeginBuildTargetSelectionGrouping();


            EditorGUILayout.Space();

            EditorGUILayout.BeginVertical(GUILayout.ExpandWidth(true));

            if (EditorApplication.isPlayingOrWillChangePlaymode)
            {
                EditorGUILayout.HelpBox("Pimax XR settings cannot be changed when the editor is in play mode.", MessageType.Info);
                EditorGUILayout.Space();
            }

            EditorGUI.BeginDisabledGroup(EditorApplication.isPlayingOrWillChangePlaymode);
            if (selectedBuildTargetGroup == BuildTargetGroup.Android)
            {
                EditorGUILayout.PropertyField(stereoRenderingMode, guiStereoRenderingMode);
                EditorGUILayout.PropertyField(protectedContext, guiProtectedContext);
                EditorGUILayout.PropertyField(eyeTracking, guiEyeTracking);
            }
            EditorGUI.EndDisabledGroup();

            EditorGUILayout.EndVertical();


            EditorGUILayout.EndBuildTargetSelectionGrouping();

            serializedObject.ApplyModifiedProperties();
        }
    }
}
