Shader "Law/Outline"
{
    Properties
    {
        _MainColor ("Main Color", Color) = (1, 1, 1, 1)
        _MainTex ("Main Tex", 2D) = "white" {}
        _AlphaScale ("Alpha Scale", Range(0, 1)) = 1
        _OutlineWidth("Outline Width", Range(0, 1)) = 0.1
        _OutlineColor("Outline Color", Color) = (1, 1, 1, 1)
    }
    SubShader
    {
        Tags
        {
            "Queue"="Transparent" "IgnoreProjector"="True" "RenderType"="Transparent"
        }

        // 只写入深度，没颜色
        Pass
        {
            ZWrite On
            ColorMask 0 //colormaskrgba ,0代表不输出颜色
        }

        //写入颜色
        Pass
        {
            Tags
            {
                "LightMode"="ForwardBase"
            }

            ZWrite Off
            Blend SrcAlpha OneMinusSrcAlpha

            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag

            #include "Lighting.cginc"

            fixed4 _MainColor;
            sampler2D _MainTex;
            float4 _MainTex_ST;
            fixed _AlphaScale;

            struct a2v
            {
                float4 vertex : POSITION;
                float3 normal : NORMAL;
                float4 texcoord : TEXCOORD0;
            };

            struct v2f
            {
                float4 pos : SV_POSITION;
                float3 worldNormal : TEXCOORD0;
                float3 worldPos : TEXCOORD1;
                float2 uv : TEXCOORD2;
            };

            v2f vert(a2v v)
            {
                v2f o;
                o.pos = UnityObjectToClipPos(v.vertex);
                o.worldNormal = UnityObjectToWorldNormal(v.normal);
                o.worldPos = mul(unity_ObjectToWorld, v.vertex).xyz;
                o.uv = TRANSFORM_TEX(v.texcoord, _MainTex);

                return o;
            }

            fixed4 frag(v2f i) : SV_Target
            {
                fixed4 texColor = tex2D(_MainTex, i.uv);
                fixed3 albedo = texColor.rgb * _MainColor.rgb;
                return fixed4(albedo.r, albedo.g, albedo.b, texColor.a * +_AlphaScale); //返回颜色混合
            }
            ENDCG
        }

        //写入描边
        Pass
        {
            Cull Front //剔除正面 
            ZWrite Off
            Blend SrcAlpha OneMinusSrcAlpha
            CGPROGRAM
            #pragma vertex vert
            #pragma fragment frag
            #include "UnityCG.cginc"


            sampler2D _MainTex;
            float4 _MainTex_ST;
            fixed _AlphaTest;
            float _OutlineWidth;
            float4 _OutlineColor;

            struct a2v
            {
                float4 vertex : POSITION;
                float4 normal : NORMAL;
                float4 texcoord : TEXCOORD0;
            };

            struct v2f
            {
                float4 pos : SV_POSITION;
                float2 uv : TEXCOORD2;
            };

            v2f vert(a2v v)
            {
                v2f o;
                float4 pos = mul(UNITY_MATRIX_MV, v.vertex);
                float3 pos2 = mul((float3x3)UNITY_MATRIX_IT_MV, v.normal);
                pos2.z = -0.5; // viewNormal 对顶点法线的z分量进行处理，使他们等于一个定值
                float4 noraml = float4(normalize(pos2), 0);
                pos = pos + noraml * _OutlineWidth; // viewPos 沿法线方向对顶点进行扩大
                o.pos = mul(UNITY_MATRIX_P, pos);
                o.uv = TRANSFORM_TEX(v.texcoord, _MainTex);
                return o;
            }


            fixed4 frag(v2f i) : SV_Target
            {
                fixed4 texColor = tex2D(_MainTex, i.uv);
                return fixed4(_OutlineColor.r, _OutlineColor.g, _OutlineColor.b, texColor.a); //根据描边渲染颜色
            }
            ENDCG
        }
    }
    FallBack "Transparent/VertexLit"
}