using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR;

namespace Unity.XR.Pimax
{
    public class PvrControllerEffects : MonoBehaviour
    {

        private Animator ControllerAnimator;
        public PvrController hand;
        private XRNode node;
        private Vector2 primary2DAxis;
        private bool Primary2DAxisTouch, MenuButton, PrimaryButton, SecondaryButton;
        private float Trigger, Grip;
        private InputDevice device;
        private bool tracked;

        void Start()
        {
            ControllerAnimator = GetComponent<Animator>();
            switch (hand)
            {
                case PvrController.LeftController:
                    node = XRNode.LeftHand;
                    break;
                case PvrController.RightController:
                    node = XRNode.RightHand;
                    break;
            }
            device = InputDevices.GetDeviceAtXRNode(node);
        }

        void Update()
        {
            //根据当前输入设置动画器变量
            GetButtonState(node);
            ChangeKeyEffects();
            //未追踪时隐藏模型
            if (!device.TryGetFeatureValue(CommonUsages.isTracked, out tracked))
            {
                return;
            }
            MeshRenderer[] meshRenderers = transform.GetComponentsInChildren<MeshRenderer>();
            foreach (MeshRenderer meshRenderer in meshRenderers)
            {
                meshRenderer.enabled = tracked;
            }
        }

        private void GetButtonState(XRNode node)
        {
            device.TryGetFeatureValue(CommonUsages.primary2DAxisTouch, out Primary2DAxisTouch);
            device.TryGetFeatureValue(CommonUsages.menuButton, out MenuButton);
            device.TryGetFeatureValue(CommonUsages.grip, out Grip);
            device.TryGetFeatureValue(CommonUsages.trigger, out Trigger);
            device.TryGetFeatureValue(CommonUsages.primaryButton, out PrimaryButton);
            device.TryGetFeatureValue(CommonUsages.secondaryButton, out SecondaryButton);
            device.TryGetFeatureValue(CommonUsages.primary2DAxis, out primary2DAxis);
        }

        private void ChangeKeyEffects()
        {
            if (ControllerAnimator == null)
            {
                return;
            }
            ControllerAnimator.SetFloat("x", primary2DAxis.x);
            ControllerAnimator.SetFloat("y", primary2DAxis.y);
            ControllerAnimator.SetFloat("Primary2DAxisTouch", Primary2DAxisTouch ? 1.0f : 0.0f);
            ControllerAnimator.SetFloat("MenuButton", MenuButton ? 1.0f : 0.0f);
            ControllerAnimator.SetFloat("Grip", Grip);
            ControllerAnimator.SetFloat("Trigger", Trigger);
            ControllerAnimator.SetFloat("PrimaryButton", PrimaryButton ? 1.0f : 0.0f);
            ControllerAnimator.SetFloat("SecondaryButton", SecondaryButton ? 1.0f : 0.0f);
        }
    }
}

