using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System;
using UnityEngine;
using UnityEngine.XR;

namespace Unity.XR.Pimax
{
    public class PvrInput
    {

        /// <summary>
        /// Get the current main controller.
        /// </summary>
        /// <returns>
        /// Left or Right Controller.
        /// </returns>
        public static PvrController GetDominantHand()
        {
            return 0;
        }

        /// <summary>
        /// Set the current main controller.
        /// </summary>
        /// <param name="controller">Left or Right Controller.</param>
        public static void SetDominantHand(PvrController controller)
        {
            //PvrPlugin.pvr_SetControllerMainInputHandle((UInt32)controller);
        }

        /// <summary>
        /// Set controller vibrate.
        /// </summary>
        /// <param name="strength">vibrate strength.</param>
        /// <param name="time">vibrate time.</param>
        /// <param name="controller">Left or Right Controller.</param>
        public static void SetControllerVibration(float strength, int time, PvrController controller)
        {
            PvrPlugin.pvr_ControllerSendMessage((int)controller, 1, (int)strength, time);
        }

        /// <summary>
        /// Check whether controller is connected.
        /// </summary>
        /// <param name="controller">Left or Right Controller</param>
        /// <returns>
        ///   <c>true</c> if the controller connected; otherwise, <c>false</c>.
        /// </returns>
        public static bool IsControllerConnected(PvrController controller)
        {
            sxrControllerState controllerState = PvrPlugin.pvr_ControllerGetState((int)controller);
            bool connectionState = controllerState.connectionState == sxrControllerConnectionState.kConnected;
            return connectionState;
        }

        /// <summary>
        /// Set the controller origin offset.
        /// </summary>
        /// <param name="controller">Left or Right Controller.</param>
        /// <param name="offset">m</param>
        public static void SetControllerOriginOffset(PvrController controller, Vector3 offset)
        {
            //PvrPlugin.pvr_SetControllerOriginOffset((int)controller, offset);
        }

        /// <summary>
        /// Get the predict controller rotation after specified time.
        /// </summary>
        /// <param name="controller">Left or Right Controller.</param>
        /// <param name="predictTime">ms</param>
        /// <returns>
        /// Quaternion controller predict rotation.
        /// </returns>
        public static Quaternion GetControllerPredictRotation(PvrController controller, double predictTime)
        {
            PvrControllerTracking pvrControllerTracking = new PvrControllerTracking();

            //PvrPlugin.pvr_GetControllerTrackingState((uint)controller, predictTime, ref pvrControllerTracking);

            return new Quaternion(
                pvrControllerTracking.localControllerPose.pose.orientation.x,
                pvrControllerTracking.localControllerPose.pose.orientation.y,
                pvrControllerTracking.localControllerPose.pose.orientation.z,
                pvrControllerTracking.localControllerPose.pose.orientation.w);
        }

        /// <summary>
        /// Get the predict controller rotation after specified time.
        /// </summary>
        /// <param name="controller">Left or Right Controller.</param>
        /// <param name="predictTime">ms</param>
        /// <returns>
        /// Vector3 controller predict position.
        /// </returns>
        public static Vector3 GetControllerPredictPosition(PvrController controller, double predictTime)
        {
            PvrControllerTracking pvrControllerTracking = new PvrControllerTracking();

            //PvrPlugin.pvr_GetControllerTrackingState((uint)controller, predictTime, ref pvrControllerTracking);

            return new Vector3(
                pvrControllerTracking.localControllerPose.pose.position.x,
                pvrControllerTracking.localControllerPose.pose.position.y,
                pvrControllerTracking.localControllerPose.pose.position.z);
        }

        public static PvrControllerType GetControllerDeviceType(PvrController controller)
        {
            IntPtr ptr = PvrPlugin.pvr_GetControllerDeviceType();
            if (ptr == IntPtr.Zero)
            {
                return PvrControllerType.H1;
            }
            int arrayLength = 2;
            int[] controllerType = new int[arrayLength];
            Marshal.Copy(ptr, controllerType, 0, arrayLength);


            if (controller == PvrController.LeftController)
            {
                return (PvrControllerType)controllerType[0];
            }
            else
            {
                return (PvrControllerType)controllerType[1];
            }
        }
    }
}

