using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace Unity.XR.Pimax
{
    public class PvrBoundary
    {

        /// <summary>
        /// Get the boundary enabled or not.
        /// </summary>
        /// <returns>enabled or not</returns>
        public static bool GetEnabled()
        {
            return PvrPlugin.pvr_IsBoundarySystemEnabled();
        }

        /// <summary>
        /// Set the boundary enabled or not.
        /// </summary>
        /// <param name="value">enabled or not.</param>
        public static void SetEnabled(bool value)
        {
            PvrPlugin.pvr_SetBoundaryDisable(value);
        }

        /// <summary>
        /// Gets if the boundary data is configured yet.
        /// </summary>
        /// <returns>is configured</returns>
        public static bool GetConfigured()
        {
            //return PvrPlugin.pvr_GetBoundaryConfigured();
            return false;
        }

        /// <summary>
        /// Force the visibility of boundary.
        /// </summary>
        /// <param name="value">the value will be set.</param>
        public static void SetVisible(bool value)
        {
            PvrPlugin.pvr_RequestBoundaryVisible(value);
        }

        /// <summary>
        /// Gets the visibility of boundary.
        /// </summary>
        /// <returns>visibility of boundary</returns>
        public static bool GetVisiable()
        {
            //return PvrPlugin.pvr_GetBoundaryVisiable();
            return false;
        }

        /// <summary>
        /// Test whether the node trigger the boundary.
        /// </summary>
        /// <param name="node">The node.</param>
        /// <param name="boundaryType">Type of the boundary.</param>
        /// <returns>
        /// test result
        /// </returns>
        public static bool TestNode(BoundaryTrackingNode node, BoundaryType boundaryType)
        {
            //return PvrPlugin.pvr_TestNodeIsInBoundary(node, boundaryType);
            return false;
        }

        /// <summary>
        /// Test whether the point trigger the boundary.
        /// </summary>
        /// <param name="point">The point.</param>
        /// <param name="boundaryType">The boundary.</param>
        /// <returns>
        /// test result
        /// </returns>
        public static bool TestPoint(sxrVector3 point, BoundaryType boundaryType)
        {
            //    return PvrPlugin.pvr_TestPointIsInBoundary(point, boundaryType);
            return false;
        }

        /// <summary>
        /// Gets the points on the boundary.
        /// </summary>
        /// <param name="boundaryType">Type of the boundary.</param>
        /// <returns>
        /// collection of the points
        /// </returns>
        public static Vector3[] GetGeometry(BoundaryType boundaryType)
        {
            sxrVector3[] outPoints;
            outPoints = new sxrVector3[0];
            Vector3[] points = new Vector3[outPoints.Length];

            for (int i = 0; i < outPoints.Length; i++)
            {
                points[i] = new Vector3() 
                {
                    x = outPoints[i].x,
                    y = outPoints[i].y,
                    z = outPoints[i].z,
                };
            }
            return points;
        }

        /// <summary>
        /// Gets the dimension of the play area.
        /// </summary>
        /// <param name="boundaryType">Type of the boundary.</param>
        /// <returns>
        /// Vector3 play area dimension
        /// </returns>
        public static Vector3 GetDimensions(BoundaryType boundaryType)
        {
            sxrVector3 dimension = new sxrVector3();
            dimension = new sxrVector3();
            return new Vector3(dimension.x, dimension.y, dimension.z);
        }

        /// <summary>
        /// Get Boundary Dialog State.
        /// </summary>
        /// <returns>
        /// 
        /// </returns>
        public static int GetDialogState()
        {
            //return PvrPlugin.pvr_GetDialogState();
            return 0;
        }

        /// <summary>
        /// Use camera image as scene background
        /// </summary>
        /// <param name="value">enable or disable SeeThrough.</param>
        /// <returns>success or failed</returns>
        public static bool EnableSeeThroughManual(bool value)
        {
            SxrResult result = PvrPlugin.pvr_SetSeeThroughBackground(value);
            if (result == SxrResult.SXR_ERROR_NONE)
            {
                return true;
            }
            PvrLog.LogError(string.Format("Failed at SetTrackingSpace code: {0}", result));
            return false;
        }
    }
}