using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Runtime.InteropServices;
using System;

namespace Unity.XR.Pimax
{
    public class PvrSystem
    {


        /// <summary>
        /// Get the current SDK Version.
        /// </summary>
        /// <returns>
        /// string SDK version
        /// </returns>
        public static string GetSDKVersion()
        {
            IntPtr intPtr = PvrPlugin.pvr_GetSDKVersion();
            if (intPtr == IntPtr.Zero)
            {
                PvrLog.LogError("GetSDKVersion Faileds");
                return "0";
            }
            string SDKVersion = Marshal.PtrToStringAnsi(intPtr);
            Marshal.FreeHGlobal(intPtr);
            return SDKVersion;
        }

        /// <summary>
        /// Get current display frequency of the screen.
        /// </summary>
        /// <returns>
        /// float display frequency
        /// </returns>
        public static float GetSystemDisplayFrequency()
        {
            sxrDeviceInfo deviceInfo = PvrPlugin.pvr_GetDeviceInfo();
            return deviceInfo.displayRefreshRateHz;
        }

        /// <summary>
        /// Gets the predicted time it takes to display a frame after being rendered.
        /// </summary>
        /// <returns>
        /// float predicted time
        /// </returns>
        public static float GetPredictedDisplayTime()
        {
            float displayTime = PvrPlugin.pvr_GetPredictedDisplayTime();
            if (displayTime == -1.0f)
            {
                PvrLog.LogError("GetPredictedDisplayTime Failed");
            }
            return displayTime;
        }

        /// <summary>
        /// Sets the extra latency mode.
        /// </summary>
        /// <param name="mode">Latency mode.</param>
        /// <returns>
        /// Whether the extra latency mode has been set.
        /// </returns>
        public static bool SetExtraLatencyMode(pxrExtraLatencyMode mode)
        {
            SxrResult result = PvrPlugin.pvr_SetExtraLatencyMode(mode);
            if (result == SxrResult.SXR_ERROR_NONE)
            {
                return true;
            }
            PvrLog.LogError(string.Format("Failed at SetExtraLatencyMode code: {0}", result));
            return false;
        }

        /// <summary>
        /// Sets the tracking space type.
        /// </summary>
        /// <param name="whichSpace">tracking space type.</param>
        /// <returns>
        /// Whether the tracking space type has been set.
        /// </returns>
        public static bool SetTrackingSpace(pxrTrackingSpace whichSpace)
        {
            SxrResult result = PvrPlugin.pvr_SetTrackingSpace(whichSpace);
            if (result == SxrResult.SXR_ERROR_NONE)
            {
                return true;
            }
            PvrLog.LogError(string.Format("Failed at SetTrackingSpace code: {0}", result));
            return false;
        }

    }
}