﻿using System.Collections.Generic;
using UnityEngine.Experimental.XR;
using UnityEngine.XR;
using UnityEngine.XR.Management;
using UnityEngine;
using UnityEngine.SceneManagement;

#if UNITY_INPUT_SYSTEM
using UnityEngine.InputSystem;
using UnityEngine.InputSystem.Layouts;
using UnityEngine.InputSystem.XR;
using Unity.XR.Pimax.Input;
#endif

#if UNITY_EDITOR
using UnityEditor;
#endif

namespace Unity.XR.Pimax
{
#if UNITY_INPUT_SYSTEM
#if UNITY_EDITOR
    [InitializeOnLoad]
#endif
    static class InputLayoutLoader
    {
        static InputLayoutLoader()
        {
            RegisterInputLayouts();
        }

        public static void RegisterInputLayouts()
        {
            InputSystem.RegisterLayout<PimaxHMD>(
                matches: new InputDeviceMatcher()
                .WithInterface(XRUtilities.InterfaceMatchAnyVersion)
                .WithProduct(@"^(Pimax HMD)")
                );
            InputSystem.RegisterLayout<PimaxController>(
                matches: new InputDeviceMatcher()
                .WithInterface(XRUtilities.InterfaceMatchAnyVersion)
                .WithProduct(@"^(Pimax Left Hand)|^(Pimax Right Hand)")
                );
        }
    }
#endif
    public class PvrLoader : XRLoaderHelper
    {
        private static List<XRDisplaySubsystemDescriptor> s_DisplaySubsystemDescriptors =
            new List<XRDisplaySubsystemDescriptor>();
        private static List<XRInputSubsystemDescriptor> s_InputSubsystemDescriptors =
            new List<XRInputSubsystemDescriptor>();

        public XRDisplaySubsystem displaySubsystem
        {
            get
            {
                return GetLoadedSubsystem<XRDisplaySubsystem>();
            }
        }

        public XRInputSubsystem inputSubsystem
        {
            get
            {
                return GetLoadedSubsystem<XRInputSubsystem>();
            }
        }

        public override bool Initialize()
        {
            PvrLog.LogInfo("Pimax Mobile XR SDK v0.6.2: start init");
#if UNITY_INPUT_SYSTEM
            InputLayoutLoader.RegisterInputLayouts();
#endif
#if UNITY_ANDROID
            PvrSettings settings = PvrSettings.instance;
            if (settings != null)
            {
                PimaxXRSettings pimaxXRSettings = new PimaxXRSettings();
                pimaxXRSettings.StereoRenderingMode = settings.stereoRenderingMode;
                pimaxXRSettings.colorSpace = QualitySettings.activeColorSpace;
                pimaxXRSettings.protectedContext = settings.protectedContext;

                //PvrPlugin.pvr_SetXRSettings(pimaxXRSettings);
            }
#endif
            CreateSubsystem<XRDisplaySubsystemDescriptor, XRDisplaySubsystem>(s_DisplaySubsystemDescriptors, "Pimax Display");
            CreateSubsystem<XRInputSubsystemDescriptor, XRInputSubsystem>(s_InputSubsystemDescriptors, "Pimax Input");

            if (displaySubsystem == null || inputSubsystem == null)
            {
                PvrLog.LogInfo("Unable to start Pimax XR Plugin.");
            }

            if (displaySubsystem == null)
            {
                PvrLog.LogInfo("Failed to load display subsystem.");
            }

            if (inputSubsystem == null)
            {
                PvrLog.LogInfo("Failed to load input subsystem.");
            }

            return true;
        }

        public override bool Start()
        {
            PvrLog.LogInfo("start StartSubsystem");

            StartSubsystem<XRDisplaySubsystem>();
            StartSubsystem<XRInputSubsystem>();

            //SceneManager.sceneLoaded += Recenter;
            return true;
        }

        public override bool Stop()
        {
            PvrLog.LogInfo("start StopSubsystem");
            StopSubsystem<XRDisplaySubsystem>();
            StopSubsystem<XRInputSubsystem>();
            return true;
        }

        public override bool Deinitialize()
        {
            PvrLog.LogInfo("start DeinitializeSubsystem");
            DestroySubsystem<XRDisplaySubsystem>();
            DestroySubsystem<XRInputSubsystem>();
            return true;
        }

        void Recenter(Scene scene, LoadSceneMode loadSceneMode)
        {
            if (scene == SceneManager.GetSceneByBuildIndex(1))
            {
                PvrLog.LogInfo("TryRecenter" + inputSubsystem.TryRecenter());
                SceneManager.sceneLoaded -= Recenter;
            }
        }

    }
}
