using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR.Management;

namespace Unity.XR.Pimax
{
    public class PvrManager : MonoBehaviour
    {
        private bool isPaused = false;
        private static PvrManager instance = null;
        public static PvrManager Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = FindObjectOfType<PvrManager>();
                    if (instance == null)
                    {
                        PvrLog.LogError("PvrManager is not initialized,put the PvrManager.cs on your XR Origin or XR Rig");
                    }
                }
                return instance;
            }
        }
        public FoveationLevel foveationLevel;

        private void OnEnable()
        {
            if (foveationLevel != FoveationLevel.None)
            {
                PvrFoveationRendering.SetFoveationLevel(foveationLevel);
            }
        }

        private void Update()
        {

        }

        void OnApplicationFocus(bool hasFocus)
        {
            if (isPaused != !hasFocus)
            {
                isPaused = !hasFocus;
                StartOrStopXR(isPaused);
            }
        }

        void OnApplicationPause(bool pauseStatus)
        {
            if (isPaused != pauseStatus)
            {
                isPaused = pauseStatus;
                StartOrStopXR(isPaused);
            }
        }

        private void StartOrStopXR(bool isPaused)
        {
            if (isPaused)
            {
                XRGeneralSettings.Instance.Manager.StopSubsystems();
            }
            else
            {
                XRGeneralSettings.Instance.Manager.StartSubsystems();
            }
        }

        private void OnApplicationQuit() 
        {
            XRGeneralSettings.Instance.Manager.DeinitializeLoader();
        }
    }

}

