using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System;
using System.Runtime.InteropServices;

namespace Unity.XR.Pimax
{
    [StructLayout(LayoutKind.Sequential)]
    public struct PimaxXRSettings
    {
        public PvrSettings.StereoRenderingMode StereoRenderingMode;
        public ColorSpace colorSpace;
        public bool protectedContext;
    }
    public enum BoundaryType
    {
        OuterBoundary,
        PlayArea
    }

    public enum PvrController
    {
        LeftController,
        RightController,
    }

    public enum PvrControllerType
    {
        H1,
        H0,
        F1C,
    }

    public enum BoundaryTrackingNode
    {
        HandLeft,
        HandRight,
        Head
    }

    public enum sxrWarpMeshType
    {
        kMeshTypeColumsLtoR = 0,    // Columns Left to Right
        kMeshTypeColumsRtoL,        // Columns Right to Left
        kMeshTypeRowsTtoB,          // Rows Top to Bottom
        kMeshTypeRowsBtoT,          // Rows Bottom to Top
    }

    //[StructLayout(LayoutKind.Sequential)]
    //public struct PvrBoundaryTriggerInfo
    //{
    //    public bool isTriggering;
    //    public float closestDistance;
    //    public PvrVector3f closestPoint;
    //    public PvrVector3f closestPointNormal;
    //    public bool valid;
    //};

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrFoveation
    {
        public sxrVector2 gain;             //!< Foveation Gain Rate [1, ...]
        public float area;                  //!< Foveation Area Size [0, ...]
        public float minimum;               //!< Foveation Minimum Resolution [1, 1/2, 1/4, ..., 1/16, 0]
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct PvrControllerCapability
    {
        public PvrControllerType type;
        public PvrControllerDof inputDof;
        public PvrControllerBond inputBond;
        public UInt64 Abilities;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrDeviceInfo
    {
        public Int32 displayWidthPixels;         //!< Physical width of the display (pixels)
        public Int32 displayHeightPixels;        //!< Physical height of the display (pixels)
        public float displayRefreshRateHz;       //!< Refresh rate of the display
        public Int32 displayOrientation;         //!< Display orientation (degrees at initialization - 0,90,180,270)
        public Int32 targetEyeWidthPixels;       //!< Recommended eye buffer width (pixels)
        public Int32 targetEyeHeightPixels;      //!< Recommended eye buffer height (pixels)
        public float targetFovXRad;              //!< Recommended horizontal FOV
        public float targetFovYRad;              //!< Recommended vertical FOV
        public sxrViewFrustum leftEyeFrustum;             //!< Recommended Frustum information for left eye
        public sxrViewFrustum rightEyeFrustum;            //!< Recommended Frustum information for right eye
        public float targetEyeConvergence;       //!< Recommended eye convergence
        public float targetEyePitch;             //!< Recommended eye pitch
        public Int32 deviceOSVersion;            //!< Android OS Version of the device
        public sxrWarpMeshType warpMeshType;               //!< Type of mesh used to render eye buffer
        public sxrFoveation lowFoveation;               //!< Low foveation values
        public sxrFoveation medFoveation;               //!< Medium foveation values
        public sxrFoveation highFoveation;              //!< High foveation values

        // Tracking Camera
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 12)]
        public float[] trackingCalibration;    //!< Tracking Camera calibration 4x3 matrix
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
        public float[] trackingPrincipalPoint;  //!< Tracking Camera principal point
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 2)]
        public float[] trackingFocalLength;     //!< Tracking Camera focal length
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        public float[] trackingDistortion;      //!< Tracking Camera radial distortion

        // Tracking capabilities
        public UInt64 trackingCapabilities;       //!< Qvr service capabilities: Eye Tracking

        public Int32 deviceType;                 //!< Device name
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrViewFrustum
    {
        public float left;           //!< Left Plane of Frustum
        public float right;          //!< Right Plane of Frustum
        public float top;            //!< Top Plane of Frustum
        public float bottom;         //!< Bottom Plane of Frustum

        public float near;           //!< Near Plane of Frustum
        public float far;            //!< Far Plane of Frustum (Arbitrary)

        public sxrVector3 position;       //!< Position Offset of Frustum
        public sxrQuaternion rotation;       //!< Rotation Quaternion of Frustum
    }
    
    public enum PvrControllerDof
    {
        Controller3Dof,
        Controller6Dof
    }

    public enum PvrControllerBond
    {
        ControllerIsBond,
        ControllerUnBond
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct PvrSensorState
    {
        public int status;
        public PvrPosef pose;
        public sxrVector3 angularVelocity;
        public sxrVector3 linearVelocity;
        public sxrVector3 angularAcceleration;
        public sxrVector3 linearAcceleration;
        public UInt64 poseTimeStampNs;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct PvrPosef
    {
        public sxrQuaternion orientation;
        public sxrVector3 position;
    };

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrQuaternion
    {
        public float x;
        public float y;
        public float z;
        public float w;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrVector3
    {
        public float x;
        public float y;
        public float z;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrVector2
    {
        public float x;
        public float y;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct PvrControllerTracking
    {
        public PvrSensorState localControllerPose;
        public PvrSensorState globalControllerPose;
    }

    public enum pxrSystemPropertyType
    {
        PXRAPI_SYS_PROP_DEVICE_TYPE = 0,
        PXRAPI_SYS_PROP_DISPLAY_PIXELS_WIDE = 1,
        PXRAPI_SYS_PROP_DISPLAY_PIXELS_HIGH = 2,
        PXRAPI_SYS_PROP_DISPLAY_REFRESH_RATE = 3,               // ʵʱ֡��
        PXRAPI_SYS_PROP_SUGGESTED_EYE_TEXTURE_WIDTH = 4,
        PXRAPI_SYS_PROP_SUGGESTED_EYE_TEXTURE_HEIGHT = 5,
        PXRAPI_SYS_PROP_SUGGESTED_EYE_FOV_DEGREES_X = 6,
        PXRAPI_SYS_PROP_SUGGESTED_EYE_FOV_DEGREES_Y = 7,
        PXRAPI_SYS_PROP_DISPLAY_ORIENTATION = 8,
        PXRAPI_SYS_PROP_DEVICE_OS_VERSION = 9,
        PXRAPI_SYS_PROP_HAS_ORIENTATION_TRACKING = 10,
        PXRAPI_SYS_PROP_HAS_POSITION_TRACKING = 11,
        PXRAPI_SYS_PROP_HAS_EYE_TRACKING = 12,
        PXRAPI_SYS_PROP_SUPPORTED_DISPLAY_REFRESH_RATES = 13,   // ����֧�ֵ�ˢ��������
        PXRAPI_SYS_PROP_NUM_SUPPORTED_SWAPCHAIN_FORMATS = 14,   //TODO: ��֪����ʲô��
        PXRAPI_SYS_PROP_SUPPORTED_SWAPCHAIN_FORMATS = 15,       //TODO: ��֪����ʲô��
        PXRAPI_SYS_PROP_FOVEATION_AVAILABLE = 16,               // ע�ӵ���Ⱦ�Ŀ��أ���������������һ������
    }

    public enum SxrResult
    {
        // Return code for success
        SXR_ERROR_NONE = 0,

        // Return code for cases that don't fall into defined error codes
        SXR_ERROR_UNKNOWN = 1,

        // Return code to indicate that SXR isn't supported, or the necessary functionality in the system drivers
        // or VR service isn't supported.
        SXR_ERROR_UNSUPPORTED,

        // Return code for calls to functions made without first calling sxrInitialize
        SXR_ERROR_VRMODE_NOT_INITIALIZED,

        // Return code for calls made into functions that require VR mode to be in a started state
        SXR_ERROR_VRMODE_NOT_STARTED,

        // Return code for calls made into functions that require VR mode to be in a stopped state
        SXR_ERROR_VRMODE_NOT_STOPPED,

        // Return code for calls made into functions where the service fails or isn't accessible
        SXR_ERROR_QVR_SERVICE_UNAVAILABLE,

        // Error for any failures in JNI/Java calls made through the API
        SXR_ERROR_JAVA_ERROR,
    }

    public enum pxrPropertyType
    {
        PXRAPI_FOVEATION_LEVEL = 0,
    }

    public enum pxrExtraLatencyMode
    {
        PXRAPI_EXTRA_LATENCY_MODE_OFF = 0,
        PXRAPI_EXTRA_LATENCY_MODE_ON = 1,
        PXRAPI_EXTRA_LATENCY_MODE_DYNAMIC = 2
    }

    public enum pxrTrackingSpace
    {
        PXRAPI_TRACKING_SPACE_LOCAL = 0, // Eye level origin - controlled by system recentering
        PXRAPI_TRACKING_SPACE_LOCAL_FLOOR = 1, // Floor level origin - controlled by system recentering
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct sxrEyePoseState
    {
        public Int64 timestamp;                  //!< Timestamp for the eye pose

        public Int32 leftEyePoseStatus;          //!< Bit field (sxrEyePoseStatus) indicating left eye pose status
        public Int32 rightEyePoseStatus;         //!< Bit field (sxrEyePoseStatus) indicating right eye pose status
        public Int32 combinedEyePoseStatus;      //!< Bit field (sxrEyePoseStatus) indicating combined eye pose status

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] leftEyeGazePoint;        //!< Left Eye Gaze Point
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] rightEyeGazePoint;       //!< Right Eye Gaze Point
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] combinedEyeGazePoint;    //!< Combined Eye Gaze Point (HMD center-eye point)

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] leftEyeGazeVector;       //!< Left Eye Gaze Point
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] rightEyeGazeVector;      //!< Right Eye Gaze Point
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] combinedEyeGazeVector;   //!< Comnbined Eye Gaze Vector (HMD center-eye point)

        public bool leftEyeBlink;               //!< Left eye blink status (true = eye is closed, false = eye is open)
        public bool rightEyeBlink;              //!< Right eye blink status (true = eye is closed, false = eye is open)

        public float leftEyeOpenness;            //!< Left eye value between 0.0 and 1.0 where 1.0 means fully open and 0.0 closed.
        public float rightEyeOpenness;           //!< Right eye value between 0.0 and 1.0 where 1.0 means fully open and 0.0 closed.

        public float leftEyePupilDilation;       //!< Left eye value in millimeters indicating the pupil dilation
        public float rightEyePupilDilation;      //!< Right eye value in millimeters indicating the pupil dilation

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] leftEyePositionGuide;    //!< Position of the inner corner of the left eye in meters from the HMD center-eye coordinate system's origin.
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] rightEyePositionGuide;   //!< Position of the inner corner of the right eye in meters from the HMD center-eye coordinate system's origin.

        public int foveatedEyeGazeState;       //!< svrFoveatedEyeGazeState
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 3)]
        public float[] foveatedEyeGazeVector;   //!< Foveated Eye Gaze Vector (HMD center-eye point)
    }

    public struct sxrControllerState
    {
        //! Orientation
        public sxrQuaternion rotation;

        //! Position
        public sxrVector3 position;

        //! Gyro
        public sxrVector3 gyroscope;

        //! Accelerometer
        public sxrVector3 accelerometer;

        //! timestamp
        public UInt64 timestamp;

        //! All digital button states as bitflags
        public UInt32 buttonState;   //!< all digital as bit flags

        //! Touchpads, Joysticks
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 4)]
        public sxrVector2[] analog2D;       //!< analog 2D's

        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 8)]
        //! Triggers
        public float[] analog1D;       //!< analog 1D's

        //! Whether the touchpad area is being touched.
        public UInt32 isTouching;

        //! Controller Connection state
        public sxrControllerConnectionState connectionState;

        //! bPosValid
        public bool bPosValid;     //check tracking lost?

        //! Velocity
        public sxrVector3 velocity;
    }

    public enum sxrControllerConnectionState
    {
        kNotInitialized = 0,
        kDisconnected = 1,
        kConnected = 2,
        kConnecting = 3,
        kError = 4
    }



    /// <summary>
    /// 
    /// </summary>
    public class PvrPlugin
    {
        const string PvrDLL = "PimaxXRPlugin";

        //controller
        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern void pvr_SetControllerOriginOffset(int controllerID, Vector3 offset);

        ////[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        ////public static extern int pvr_GetControllerTrackingState(UInt32 deviceID, double predictTime, ref PvrControllerTracking tracking);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern int pvr_SetControllerMainInputHandle(UInt32 deviceID);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern int pvr_GetControllerMainInputHandle(ref int deviceID);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern sxrControllerState sxrControllerGetState(int controllerHandle, int controllerSpace = 0);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void sxrControllerSendMessage(int controllerHandle, int what, int arg1, int arg2);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr GetControllerDeviceType();

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern int pvr_GetControllerCapabilities(UInt32 deviceID, ref PvrControllerCapability capability);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern sxrControllerState sxrControllerGetState(int controllerHandle, int controllerSpace = 0);

        //boundary
        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern bool sxrIsBoundarySystemEnabled();

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrSetBoundaryDisable(bool value);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern SxrResult pxrSetSeeThroughBackground(bool value);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrRequestBoundaryVisible(bool value);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern bool pvr_GetBoundaryVisiable();

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern PvrBoundaryTriggerInfo pvr_TestNodeIsInBoundary(BoundaryTrackingNode node, BoundaryType boundaryType);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern PvrBoundaryTriggerInfo pvr_TestPointIsInBoundary(PvrVector3f node, BoundaryType boundaryType);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern PvrVector3f[] pvr_GetBoundaryGeometry(BoundaryType boundaryType);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern PvrVector3f pvr_GetBoundaryDimensions(BoundaryType boundaryType);

        //[DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        //public static extern int pvr_GetDialogState();

        //System
        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern int pxrGetSystemPropertyInt(pxrSystemPropertyType propType);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern float pxrGetSystemPropertyFloat(pxrSystemPropertyType propType);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern int pxrGetSystemPropertyFloatArray(pxrSystemPropertyType propType, ref float[] values, int numArrayValues);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern int pxrGetSystemPropertyInt64Array(pxrSystemPropertyType propType, ref int[] values, int numArrayValues);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr pxrGetSystemPropertyString(pxrSystemPropertyType propType);
        //
        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrSetPropertyInt(pxrPropertyType propType, int intVal);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrGetPropertyInt(pxrPropertyType propType, ref int intVal);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrSetPropertyFloat(pxrPropertyType propType, float floatVal);
        //
        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern IntPtr sxrGetVersion();

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern sxrDeviceInfo sxrGetDeviceInfo();

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern float sxrGetPredictedDisplayTime();

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern SxrResult pxrSetExtraLatencyMode(pxrExtraLatencyMode mode);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern SxrResult pxrSetTrackingSpace(pxrTrackingSpace whichSpace);

        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern void pxrSetXRSettings(PimaxXRSettings pimaxXRSettings);

        //EyeTracking
        [DllImport(PvrDLL, CallingConvention = CallingConvention.Cdecl)]
        private static extern SxrResult sxrGetEyePose(ref sxrEyePoseState pReturnPose, Int32 flag = 0);


        //--------------------------------------------------------------------------------------------


        public static void pvr_SetXRSettings(PimaxXRSettings pimaxXRSettings)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            try
            {
                pxrSetXRSettings(pimaxXRSettings);
            }
            catch (Exception e)
            {
                PvrLog.LogError(string.Format("SetXRSettings Failed code: {0}", e));
            }
#endif
        }

        public static sxrControllerState pvr_ControllerGetState(int controllerHandle, int controllerSpace = 0)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return sxrControllerGetState(controllerHandle, controllerSpace);
#else
            return new sxrControllerState();
#endif
        }

        public static SxrResult pvr_SetSeeThroughBackground(bool value)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return pxrSetSeeThroughBackground(value);
#else
            return SxrResult.SXR_ERROR_UNKNOWN;
#endif
        }

        public static void pvr_ControllerSendMessage(int controllerHandle, int what, int arg1, int arg2)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            sxrControllerSendMessage(controllerHandle, what, arg1, arg2);
#endif
        }

        public static bool pvr_IsBoundarySystemEnabled()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return sxrIsBoundarySystemEnabled();
#else
            return false;
#endif
        }

        public static void pvr_RequestBoundaryVisible(bool value)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            pxrRequestBoundaryVisible(value);
#endif
        }

        public static void pvr_SetBoundaryDisable(bool value)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            pxrSetBoundaryDisable(value);
#endif
        }

        public static int pvr_GetSystemPropertyInt(pxrSystemPropertyType propType)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return pxrGetSystemPropertyInt(propType);
#else
            return 0;
#endif
        }

        public static float pvr_GetSystemPropertyFloat(pxrSystemPropertyType propType)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return pxrGetSystemPropertyFloat(propType);
#else
            return 0.0f;
#endif
        }

        public static int pvr_GetSystemPropertyFloatArray(pxrSystemPropertyType propType, ref float[] values, int numArrayValues)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return pxrGetSystemPropertyFloatArray(propType, ref values, numArrayValues);
#else
            return 0;
#endif
        }

        public static int pvr_GetSystemPropertyInt64Array(pxrSystemPropertyType propType, ref int[] values, int numArrayValues)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return pxrGetSystemPropertyInt64Array(propType, ref values, numArrayValues);
#else
            return 0;
#endif
        }

        public static IntPtr pvr_GetSystemPropertyString(pxrSystemPropertyType propType)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            try
            {
                IntPtr intPtr = pxrGetSystemPropertyString(propType);
                return intPtr;
            }
            catch (Exception e)
            {
                PvrLog.LogError(string.Format("GetSystemPropertyString Failed code: {0}", e));
                return IntPtr.Zero;
            }
#else
            return IntPtr.Zero;
#endif
        }

        public static void pvr_SetPropertyInt(pxrPropertyType propType, int intVal)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            pxrSetPropertyInt(propType, intVal);
#endif
        }

        public static void pvr_SetPropertyFloat(pxrPropertyType propType, float floatVal)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            pxrSetPropertyFloat(propType, floatVal);
#endif
        }

        public static void pvr_GetPropertyInt(pxrPropertyType propType, ref int intVal)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            pxrGetPropertyInt(propType, ref intVal);
#endif
        }

        public static IntPtr pvr_GetSDKVersion()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            try
            {
                IntPtr intPtr = sxrGetVersion();
                return intPtr;
            }
            catch (Exception e)
            {
                PvrLog.LogError(string.Format("GetSDKVersion Failed code: {0}", e));
                return IntPtr.Zero;
            }
#else
            return IntPtr.Zero;
#endif
        }

        public static sxrDeviceInfo pvr_GetDeviceInfo()
        {
#if UNITY_ANDROID && !UNITY_EDITOR

            return sxrGetDeviceInfo();
#else
            return new sxrDeviceInfo();
#endif
        }

        public static SxrResult pvr_SetExtraLatencyMode(pxrExtraLatencyMode mode)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            SxrResult sxrResult;
            try
            {
                sxrResult = pxrSetExtraLatencyMode(mode);
                return sxrResult;
            }
            catch (Exception e)
            {
                PvrLog.LogError(string.Format("SetExtraLatencyMode Failed code: {0}", e));
                return SxrResult.SXR_ERROR_UNKNOWN;
            }
#else
            return SxrResult.SXR_ERROR_UNKNOWN;
#endif
        }

        public static SxrResult pvr_GetEyePose(ref sxrEyePoseState pReturnPose, Int32 flag = 0)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            SxrResult sxrResult;
            sxrResult = sxrGetEyePose(ref pReturnPose, flag);
            return sxrResult;
#else
            return SxrResult.SXR_ERROR_UNKNOWN;
#endif
        }

        public static SxrResult pvr_SetTrackingSpace(pxrTrackingSpace whichSpace)
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            SxrResult sxrResult;
            try
            {
                sxrResult = pxrSetTrackingSpace(whichSpace);
                return sxrResult;
            }
            catch (Exception e)
            {
                PvrLog.LogError(string.Format("SetExtraLatencyMode Failed code: {0}", e));
                return SxrResult.SXR_ERROR_UNKNOWN;
            }
#else
            return SxrResult.SXR_ERROR_UNKNOWN;
#endif
        }

        public static float pvr_GetPredictedDisplayTime()
        {
#if UNITY_ANDROID && !UNITY_EDITOR

            return sxrGetPredictedDisplayTime();
#else
            return -1.0f;
#endif
        }
        public static IntPtr pvr_GetControllerDeviceType()
        {
#if UNITY_ANDROID && !UNITY_EDITOR
            return GetControllerDeviceType();
#else
            return IntPtr.Zero;
#endif
        }
    }

}

